import os

DEVELOPMENT_ASSETS_DIRECTORY_PATH = os.path.join(os.getcwd(), "assets")

APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"
COMMON_ASSETS_DIRECTORY = "Common"
REACTTS = "reactts"

# stores the paths of the reactjs template files which are loaded locally
DEV_REACTTS_TEMPLATES_PATHS_DICT = {
    "package.json-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "package.json-tpl"
    ),
    "webpack.config.js-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "webpack.config.js-tpl"
    ),
    "babel.config.json-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "babel.config.json-tpl"
    ),
    "tsconfig.json-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "tsconfig.json-tpl"
    ),
    "module.d.ts-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "module.d.ts-tpl"
    ),
    "App.tsx-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "App.tsx-tpl"
    ),
    "index.tsx-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "index.tsx-tpl"
    ),
    "App.css-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "App.css-tpl"
    ),
    "reactlogo.png": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, REACTTS, "reactlogo.png"
    ),
}

# Development template files for reactjs configuration
DEV_REACTTS_TEMPLATE_FILES = [
    (
        APP_DIRECTORY_NAME,
        "package.json",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["package.json-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "webpack.config.js",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["webpack.config.js-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "babel.config.json",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["babel.config.json-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "tsconfig.json",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["tsconfig.json-tpl"],
    ),
    (
        SRC_DIRECTORY_NAME,
        "module.d.ts",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["module.d.ts-tpl"],
    ),
    (SRC_DIRECTORY_NAME, "App.tsx", DEV_REACTTS_TEMPLATES_PATHS_DICT["App.tsx-tpl"]),
    (
        SRC_DIRECTORY_NAME,
        "index.tsx",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["index.tsx-tpl"],
    ),
    (SRC_DIRECTORY_NAME, "App.css", DEV_REACTTS_TEMPLATES_PATHS_DICT["App.css-tpl"]),
    (
        SRC_DIRECTORY_NAME,
        "reactlogo.png",
        DEV_REACTTS_TEMPLATES_PATHS_DICT["reactlogo.png"],
    ),
]



from django_webpack_dev_server.management.template_constants.production.reactjs_constants import (
    PROD_REACTJS_TEMPLATE_FILES,
)

from django_webpack_dev_server.management.template_constants.production.reactts_constants import (
    PROD_REACTTS_TEMPLATE_FILES,
)

APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"


COMMON_TEMPLATES_URLS_DICT = {
    "urls.py-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/Common/urls.py-tpl",
    "views.py-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/Common/views.py-tpl",
    "index.html-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/Common/index.html-tpl",
    "djangologo.png": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/Common/djangologo.png",
}

# Template files that will be common for any frontend library or framework
COMMON_TEMPLATE_FILES = [
    (APP_DIRECTORY_NAME, "urls.py", COMMON_TEMPLATES_URLS_DICT["urls.py-tpl"]),
    (APP_DIRECTORY_NAME, "views.py", COMMON_TEMPLATES_URLS_DICT["views.py-tpl"]),
    (
        SRC_DIRECTORY_NAME,
        "djangologo.png",
        COMMON_TEMPLATES_URLS_DICT["djangologo.png"],
    ),
    (
        TEMPLATES_DIRECTORY_NAME,
        "index.html",
        COMMON_TEMPLATES_URLS_DICT["index.html-tpl"],
    ),
]

PROD_ALL_REACTJS_TEMPLATE_FILES = COMMON_TEMPLATE_FILES + PROD_REACTJS_TEMPLATE_FILES

PROD_ALL_REACTTS_TEMPLATE_FILES = COMMON_TEMPLATE_FILES + PROD_REACTTS_TEMPLATE_FILES



from django_webpack_dev_server.management.template_constants.development import (
    development_constants,
)
from django_webpack_dev_server.management.template_constants.production import (
    production_constants,
)

# To Identify if the operating system is windows based or not
WINDOWS_OS_IDENTIFIER = "win32"

APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"

COMMON_ASSETS_DIRECTORY = "Common"

# Store the development template filenames with frontend library or framework as key
DEVELOPMENT_TEMPLATE_FILES_DICT = {
    "react_javascript": development_constants.DEV_ALL_REACTJS_TEMPLATE_FILES,
    "react_typescript": development_constants.DEV_ALL_REACTTS_TEMPLATE_FILES,
}


# Store template filenames with frontend library or framework as key
PROD_TEMPLATE_FILES_DICT = {
    "react_javascript": production_constants.PROD_ALL_REACTJS_TEMPLATE_FILES,
    "react_typescript": production_constants.PROD_ALL_REACTTS_TEMPLATE_FILES,
}

# show relevent error messages to the user
COMMAND_ERROR_MESSAGES_DICT = {
    "INVALID_APP_NAME_ERROR_MESSAGE": "App Name should be alphanumeric only",
    "SYSTEM_ERROR_MESSAGE": "Seems like node or npm is not available in your system",
    "STATICFILES_DIRS_MISSING_ERROR_MESSAGE": "STATICFILES_DIRS attribute is missing in the django settings file",
    "NPM_INSTALLATION_ERROR_MESSAGE": "There were some errors while installing dependencies",
}



import io
import os
import queue
from unittest import mock

import pytest
from django.core.management.base import CommandError
from django_webpack_dev_server.management import constants
from django_webpack_dev_server.management.generator import Generator


# A pytest fixture used to create a Generator object
@pytest.fixture
def app_generator():

    # Create the Generator object
    app_generator = Generator(
        app_name="frontend", frontend_library_or_framework="react_javascript"
    )

    # return the Generator object to the test
    return app_generator


class TestGeneratorClass:
    """
    Test Class for testing the Generator Class defined in generator.py
    """

    def test_validate_app_name(self):
        """
        Function to test the validate_app_name method of the Generator Class
        """

        # Create the Generator object and pass an invalid app_name to test
        app_generator = Generator(
            app_name="frontend@@@", frontend_library_or_framework="react_javascript"
        )

        # Check if CommandError is raised when the method is called with invalid app_name
        with pytest.raises(CommandError) as CommandErrorException:
            app_generator.validate_appname()

        # assert the message of the exception
        assert (
            str(CommandErrorException.value)
            == constants.COMMAND_ERROR_MESSAGES_DICT["INVALID_APP_NAME_ERROR_MESSAGE"]
        )

    @mock.patch("subprocess.run")
    def test_check_system_requirements(self, mocked_subprocess_run, app_generator):
        """
        Function to test the check_system_requirements method of the Generator Class
        """

        # mock the subprocess run method and set response to 0
        # Response 0 means that the command ran successfully
        mocked_subprocess_run.return_value.returncode = 0

        # call the method to test
        app_generator.check_system_requirements()

        # assert that the subprocess run method was called 2 times
        assert mocked_subprocess_run.call_count == 2

        # Reset the mock
        mocked_subprocess_run.reset_mock()

        # mock the subprocess run method and set response to 1
        # Response 1 means that the command did not ran successfully
        mocked_subprocess_run.return_value.returncode = 1

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:
            # call the method to test
            app_generator.check_system_requirements()

        # assert the message of the exception
        assert (
            str(CommandErrorException.value)
            == constants.COMMAND_ERROR_MESSAGES_DICT["SYSTEM_ERROR_MESSAGE"]
        )

        # assert that the subprocess run method was called 2 times
        assert mocked_subprocess_run.call_count == 2

    @mock.patch("os.makedirs")
    def test_create_required_directories(self, mocked_os_makedirs, app_generator):
        """
        Function to test the create_required_directories method of the Generator Class
        """

        # call the method to test
        app_generator.create_required_directories()

        # assert that the os makedirs method is called 3 times
        assert mocked_os_makedirs.call_count == 3

        # reset the mock
        mocked_os_makedirs.reset_mock()

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:
            exception_message = "Error while creating directories"

            # Raise OSError when the os makedirs method is called to
            mocked_os_makedirs.side_effect = OSError(exception_message)

            # call the method to test
            app_generator.create_required_directories()

        # assert that the os makedirs method is called 1 time
        assert mocked_os_makedirs.call_count == 1

        # assert the message of the exception
        assert str(CommandErrorException.value) == exception_message

    def test_check_if_file_is_text_document(self, app_generator):
        """
        Function to test the check_if_file_is_text_document method of the Generator Class
        """

        # file of type text document
        text_document_filename = "generator.py"

        # file of type non text document
        non_text_document_filename = "reactlogo.png"

        # assert True if file is of text document type
        assert (
            # call the method to test
            app_generator.check_if_file_is_text_document(text_document_filename)
            == True
        )

        # assert False if file is not of text document type
        assert (
            # call the method to test
            app_generator.check_if_file_is_text_document(non_text_document_filename)
            == False
        )

    def test_get_target_path_of_template_file(self, app_generator):
        """
        Function to test the get_target_path_of_template_file method of the Generator Class
        """
        filename = "webpack.config.js-tpl"

        # create the path for a while that will be stored under APP directory
        target_app_directory_filepath = os.path.join(
            os.getcwd(), app_generator.app_name
        )

        # create the path for a while that will be stored under src directory
        target_src_directory_filepath = os.path.join(
            os.getcwd(), app_generator.app_name, constants.SRC_DIRECTORY_NAME
        )

        # create the path for a while that will be stored under templates directory
        target_templates_directory_filepath = os.path.join(
            os.getcwd(),
            app_generator.app_name,
            constants.TEMPLATES_DIRECTORY_NAME,
            app_generator.app_name,
        )

        # create the final path for a particular file
        target_filepath = os.path.join(target_app_directory_filepath, filename)

        # assert the file path created and returned by the class method
        assert (
            # call the method to test
            app_generator.get_target_path_of_template_file(
                filename, constants.APP_DIRECTORY_NAME
            )
            == target_filepath
        )

        # create the final path for a particular file
        target_filepath = os.path.join(target_src_directory_filepath, filename)

        # assert the file path created and returned by the class method
        assert (
            # call the method to test
            app_generator.get_target_path_of_template_file(
                filename, constants.SRC_DIRECTORY_NAME
            )
            == target_filepath
        )

        # create the final path for a particular file
        target_filepath = os.path.join(target_templates_directory_filepath, filename)

        # assert the file path created and returned by the class method
        assert (
            # call the method to test
            app_generator.get_target_path_of_template_file(
                filename, constants.TEMPLATES_DIRECTORY_NAME
            )
            == target_filepath
        )

    @mock.patch("requests.get")
    @mock.patch("builtins.open", new_callable=mock.mock_open, read_data="My data")
    def test_download_template_files(
        self, mocked_builtins_open, mocked_requests_get, app_generator
    ):
        """
        Function to test the download_template_files method of the Generator Class
        """

        # call the method to test
        app_generator.download_template_files()

        # assert the number of times the file open is called
        assert mocked_builtins_open.call_count == 29

        # assert the number of times the get request is made
        assert mocked_requests_get.call_count == 11

        # reset the mocks
        mocked_builtins_open.reset_mock()
        mocked_requests_get.reset_mock()

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:

            exception_message = "Error while writing in the file"

            # Raise OSError during the file write operation
            mocked_builtins_open.return_value.write.side_effect = OSError(
                exception_message
            )

            # call the method to test
            app_generator.download_template_files()

        # assert the message of the exception
        assert str(CommandErrorException.value) == exception_message

        mocked_builtins_open.reset_mock()
        mocked_requests_get.reset_mock()

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:

            exception_message = "Error while writing in the file"

            # Raise OSError during the file write operation
            mocked_builtins_open.return_value.write.side_effect = [
                "No Error for 1st call",
                OSError(exception_message),
            ]

            # call the method to test
            app_generator.download_template_files()

        # assert the message of the exception
        assert str(CommandErrorException.value) == exception_message

    @mock.patch("builtins.open", new_callable=mock.mock_open, read_data="My data")
    @mock.patch("shutil.copy")
    def test_load_assets_from_local(
        self, mocked_shutil_copy, mocked_builtins_open, app_generator
    ):
        """
        Function to test the load_assets_from_local method of the Generator Class
        """

        # call the method to test
        app_generator.load_assets_from_local()

        # assert that shutil copy and open functions are called
        assert mocked_builtins_open.called == True
        assert mocked_shutil_copy.called == True

        # reset the mocks
        mocked_shutil_copy.reset_mock()
        mocked_builtins_open.reset_mock()

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:
            exception_message = "Error while writing in the file"

            # Raise OSError during the file write operation
            mocked_builtins_open.return_value.write.side_effect = OSError(
                exception_message
            )

            # call the method to test
            app_generator.load_assets_from_local()

        # assert the message of the exception
        assert str(CommandErrorException.value) == exception_message
        # assert that shutil copy and open functions are called
        assert mocked_builtins_open.called == True
        assert mocked_shutil_copy.called == True

    @mock.patch("subprocess.Popen")
    @mock.patch("os.chdir")
    def test_install_dependencies(
        self,
        mocked_os_chdir,
        mocked_subprocess_popen,
        app_generator,
    ):

        """
        Function to test the test_install_dependencies method of the Generator Class
        """

        # create a queue object which is passed to a thread
        thread_queue = queue.Queue()

        # 0 means that command ran successfully
        command_return_code = 0

        # mocked the subprocess and set the returncode value to command_return_code
        mocked_subprocess_popen.return_value.returncode = command_return_code

        # mocked the subprocess and set the stdout value to some non empty output
        mocked_subprocess_popen.return_value.stdout = io.BytesIO(b"Command\nOutput")

        # mocked the subprocess and set the return value of poll method to None and then non-empty output
        mocked_subprocess_popen.return_value.poll.side_effect = [
            None,
            "Non-empty Output",
        ]

        # call the method to test
        app_generator.install_dependencies(thread_queue)

        # assert that os chdir method is called 2 times
        assert mocked_os_chdir.call_count == 2
        # assert that subprocess popen method is called 1 time
        assert mocked_subprocess_popen.call_count == 1

        # assert that the queue receives the value from subprocess Popen
        # and value equal to the command_return_code
        assert thread_queue.get() == command_return_code

    @mock.patch("queue.Queue")
    @mock.patch("threading.Thread")
    @mock.patch("progressbar.ProgressBar.update")
    def test_install_dependencies_and_show_progress_bar(
        self,
        mocked_progressbar_ProgressBar_update,
        mocked_threading_thread,
        mocked_queue_Queue,
        app_generator,
    ):

        """
        Function to test the install_dependencies_and_show_progress_bar method of the Generator Class
        """

        # different values for thread is_alive method
        thread_is_alive_values = [True, True, True, False]

        # set the mocked threading thread is_alive method with the thread_is_alive_values list
        mocked_threading_thread.return_value.is_alive.side_effect = (
            thread_is_alive_values
        )

        # set the mocked queue with the value 0
        # 0 implies installation was successful
        mocked_queue_Queue.return_value.get.return_value = 0

        # call the method to test
        app_generator.install_dependencies_and_show_progress_bar()

        # assert that the mocked threading thread was called 1 time
        assert mocked_threading_thread.call_count == 1

        # assert that the mocked progress bar was updated to the number
        # of values present in the thread_is_alive_values list
        assert mocked_progressbar_ProgressBar_update.call_count == len(
            thread_is_alive_values
        )

        # Reset the mocks
        mocked_threading_thread.reset_mock()
        mocked_queue_Queue.reset_mock()
        mocked_progressbar_ProgressBar_update.reset_mock()

        # set the mocked threading thread is_alive method with the thread_is_alive_values list
        mocked_threading_thread.return_value.is_alive.side_effect = (
            thread_is_alive_values
        )

        # set the mocked queue with the value 1
        # 1 implies installation was not successful
        mocked_queue_Queue.return_value.get.return_value = 1

        # Check if CommandError is raised when the method is called
        with pytest.raises(CommandError) as CommandErrorException:
            # call the method to test
            app_generator.install_dependencies_and_show_progress_bar()

        # assert that the mocked threading thread was called 1 time
        assert mocked_threading_thread.call_count == 1

        # assert that the mocked progress bar was updated to the number
        # of values present in the thread_is_alive_values list
        assert mocked_progressbar_ProgressBar_update.call_count == len(
            thread_is_alive_values
        )

        # assert the message of the exception
        assert (
            str(CommandErrorException.value)
            == constants.COMMAND_ERROR_MESSAGES_DICT["NPM_INSTALLATION_ERROR_MESSAGE"]
        )

    @mock.patch.object(
        Generator, "install_dependencies_and_show_progress_bar", return_value=None
    )
    @mock.patch.object(Generator, "load_assets_from_local", return_value=None)
    @mock.patch.object(Generator, "create_required_directories", return_value=None)
    @mock.patch("django.core.management.call_command")
    @mock.patch.object(Generator, "check_system_requirements", return_value=None)
    def test_generate_in_development_mode(
        self,
        mocked_generator_check_system_requirements,
        mocked_management_call_command,
        mocked_create_required_directories,
        mocked_load_assets_from_local,
        mocked_install_dependencies_and_show_progress_bar,
        app_generator,
    ):

        """
        Function to test the generate method of the Generator Class
        """

        # call the method to test
        app_generator.generate()

        # assert that all the methods of the Generator class are called
        assert mocked_generator_check_system_requirements.called == True
        assert mocked_create_required_directories.called == True
        mocked_management_call_command.assert_called_once_with(
            "startapp", app_generator.app_name
        )
        assert mocked_load_assets_from_local.called == True
        assert mocked_install_dependencies_and_show_progress_bar.called == True

    @mock.patch.object(
        Generator, "install_dependencies_and_show_progress_bar", return_value=None
    )
    @mock.patch.object(Generator, "download_template_files", return_value=None)
    @mock.patch.object(Generator, "create_required_directories", return_value=None)
    @mock.patch("django.core.management.call_command")
    @mock.patch.object(Generator, "check_system_requirements", return_value=None)
    def test_generate_in_production_mode(
        self,
        mocked_generator_check_system_requirements,
        mocked_management_call_command,
        mocked_create_required_directories,
        mocked_download_template_files,
        mocked_install_dependencies_and_show_progress_bar,
        app_generator,
    ):

        """
        Function to test the generate method of the Generator Class
        """

        # set the environment variable SOFTWARE_ENVIRONMENT_MODE to production
        # to test that the download_template_files method is called or not

        # create the monkeypatch object
        monkeypatch = pytest.MonkeyPatch()

        # set the environment variable SOFTWARE_ENVIRONMENT_MODE to production
        monkeypatch.setenv("SOFTWARE_ENVIRONMENT_MODE", "production")

        # assert that the environment variable has changed
        assert os.environ["SOFTWARE_ENVIRONMENT_MODE"] == "production"

        # call the method to test
        app_generator.generate()

        # assert that all the methods of the Generator class are called
        assert mocked_generator_check_system_requirements.called == True
        assert mocked_create_required_directories.called == True
        mocked_management_call_command.assert_called_once_with(
            "startapp", app_generator.app_name
        )
        assert mocked_download_template_files.called == True
        assert mocked_install_dependencies_and_show_progress_bar.called == True



import os

from django_webpack_dev_server.management.template_constants.development.reactjs_constants import (
    DEV_REACTJS_TEMPLATE_FILES,
)

from django_webpack_dev_server.management.template_constants.development.reactts_constants import (
    DEV_REACTTS_TEMPLATE_FILES,
)

DEVELOPMENT_ASSETS_DIRECTORY_PATH = os.path.join(os.getcwd(), "assets")

APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"
COMMON_ASSETS_DIRECTORY = "Common"
REACTJS = "reactjs"

# Paths of the common template files which are loaded locally
DEV_COMMON_TEMPLATES_PATHS_DICT = {
    "urls.py-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "urls.py-tpl"
    ),
    "views.py-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "views.py-tpl"
    ),
    "index.html-tpl": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "index.html-tpl"
    ),
    "djangologo.png": os.path.join(
        DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "djangologo.png"
    ),
}

# Development template files which are common for any frontend library or framework
DEV_COMMON_TEMPLATE_FILES = [
    (
        APP_DIRECTORY_NAME,
        "urls.py",
        os.path.join(
            DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "urls.py-tpl"
        ),
    ),
    (
        APP_DIRECTORY_NAME,
        "views.py",
        os.path.join(
            DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "views.py-tpl"
        ),
    ),
    (
        SRC_DIRECTORY_NAME,
        "djangologo.png",
        os.path.join(
            DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "djangologo.png"
        ),
    ),
    (
        TEMPLATES_DIRECTORY_NAME,
        "index.html",
        os.path.join(
            DEVELOPMENT_ASSETS_DIRECTORY_PATH, COMMON_ASSETS_DIRECTORY, "index.html-tpl"
        ),
    ),
]

DEV_ALL_REACTJS_TEMPLATE_FILES = DEV_COMMON_TEMPLATE_FILES + DEV_REACTJS_TEMPLATE_FILES

DEV_ALL_REACTTS_TEMPLATE_FILES = DEV_COMMON_TEMPLATE_FILES + DEV_REACTTS_TEMPLATE_FILES



from django.core.management import call_command
from unittest import mock
from django_webpack_dev_server.management.generator import Generator


class TestCommand:
    """
    Test Class for testing the Command Class defined in the generate.py
    """

    @mock.patch.object(Generator, "generate")
    def test_command(self, mocked_Generator_generate):
        """
        Function to test the methods of the Command Class
        """
        # call the management command to test
        call_command("generate", "react")

        # assert that the generate method of the Generator class is called
        assert mocked_Generator_generate.called == True



APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"

# Production React Js Template files having Github Repository Links
REACTJS_TEMPLATES_URLS_DICT = {
    "package.json-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/package.json-tpl",
    "webpack.config.js-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/webpack.config.js-tpl",
    "babel.config.json-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/babel.config.json-tpl",
    "App.js-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/App.js-tpl",
    "index.js-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/index.js-tpl",
    "App.css-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/App.css-tpl",
    "reactlogo.png": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactjs/reactlogo.png",
}

# Template files specific to react js configuration
PROD_REACTJS_TEMPLATE_FILES = [
    (
        APP_DIRECTORY_NAME,
        "package.json",
        REACTJS_TEMPLATES_URLS_DICT["package.json-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "webpack.config.js",
        REACTJS_TEMPLATES_URLS_DICT["webpack.config.js-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "babel.config.json",
        REACTJS_TEMPLATES_URLS_DICT["babel.config.json-tpl"],
    ),
    (SRC_DIRECTORY_NAME, "App.js", REACTJS_TEMPLATES_URLS_DICT["App.js-tpl"]),
    (SRC_DIRECTORY_NAME, "index.js", REACTJS_TEMPLATES_URLS_DICT["index.js-tpl"]),
    (SRC_DIRECTORY_NAME, "App.css", REACTJS_TEMPLATES_URLS_DICT["App.css-tpl"]),
    (SRC_DIRECTORY_NAME, "reactlogo.png", REACTJS_TEMPLATES_URLS_DICT["reactlogo.png"]),
]



# Contains the code for the generate command

from django.core.management.base import BaseCommand
from django_webpack_dev_server.management.generator import Generator


class Command(BaseCommand):
    help = "Creates a django app which has the frontend configuration"

    def add_arguments(self, parser):

        # adding subparser for react
        subparsers = parser.add_subparsers(
            help="Command to create a django app which has the frontend configuration",
            dest="frontend_library_or_framework",
        )

        subparsers.required = True

        # add a parser for react because it can be javascript or typescript
        react_parser = subparsers.add_parser(
            "react",
            help="Command to have django app with React for Frontend Development",
        )

        language_choices = ["javascript", "typescript"]
        # add the optional app_name argument with the default value as frontend
        react_parser.add_argument(
            "--app_name", help="Frontend Django App Name", default="frontend"
        )

        # add the optional template argument with the default value as javascript
        # template argument can be either javascript or typescript
        react_parser.add_argument(
            "--template",
            help="Language for the React Project",
            default="javascript",
            choices=language_choices,
        )

    def show_success_message(self, message):
        """
        :param message: message to be shown in stdout with success style
        """
        self.stdout.write(self.style.SUCCESS(message))

    def handle(self, *args, **options):

        # get the django app_name and the frontend_library_or_framework
        # specified by the user
        app_name = options["app_name"]
        frontend_library_or_framework = options["frontend_library_or_framework"]

        # if the frontend_library_or_framework is react
        if frontend_library_or_framework == "react":
            # fetch the template argument
            template = options["template"]

            # append it to the frontend_library_or_framework
            # react and javascript -> react_javascript for React Javascript
            # react and typescript -> react_typescript for React Typescript
            frontend_library_or_framework += "_" + template

        # Create a Generator object and pass app_name and frontend_library_or_framework
        app_generator = Generator(
            app_name=app_name,
            frontend_library_or_framework=frontend_library_or_framework,
        )
        # start building the new django app
        app_generator.generate()

        # show success messages to the user after the command is successfully
        # executed
        self.show_success_message("\n\nYour App is Ready !!!")
        self.show_success_message(
            f"Add {app_name} in the INSTALLED_APPS list in the project's settings.py file"
        )
        self.show_success_message(
            f"Add {app_name}.urls in urlpatterns of the project's urls.py"
        )
        self.show_success_message(
            "Start the Django Server and cd into your app and run npm start"
        )
    


from setuptools import setup, find_packages

package_description = (
    "A Django app to setup configuration files for React in a Django Project."
)

keywords_list = [
    "django",
    "react",
    "webpack4",
    "webpack_dev_server",
    "command-line",
    "project-generator",
    "template-generator",
    "javascript",
    "typescript",
]

setup(
    name="django-webpack-dev-server",
    version="0.0.17",
    packages=find_packages(),
    license="MIT",
    author="Jiten Sidhpura",
    author_email="jitensidhpura2000@gmail.com",
    description=package_description,
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Jitensid/django-webpack-dev-server",
    download_url="https://github.com/Jitensid/django-webpack-dev-server/archive/refs/tags/0.0.17.tar.gz",
    install_requires=["requests", "progressbar2", "python-dotenv"],
    keywords=keywords_list,
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Code Generators",
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



# necessary imports
import os
import queue
import shlex
import shutil
import subprocess
import sys
import threading
from string import Template

import progressbar
import requests
from django.core import management
from django.core.management.base import CommandError

# import the constants module
from django_webpack_dev_server.management import constants
from dotenv import load_dotenv

# load the environment variables
load_dotenv()


class Generator:
    """
    Base Class to create a django app with the necessary frontend configuration
    """

    # app_name is the name of the django app which will have the frontend configuration
    app_name = None

    # static_directory_name is the name of the the static directory inside the django app
    static_directory_name = None

    # frontend_library_or_framework denotes the frontend framework or library requested from the command line
    frontend_library_or_framework = None

    # shell_parameter = True for windows based system
    # shell_parameter = False for non windows based system
    shell_parameter = None

    # set the values of app_name, frontend_library_or_framework in the constructor
    def __init__(self, app_name, frontend_library_or_framework):
        """
        Constructor to set up the app_name and frontend_library_or_framework variables of the class
        :param app_name str: Name of the django app with frontend configuration
        :param frontend_library_or_framework str: Denotes the configuration to setup in the django app.
        """
        self.app_name = app_name
        self.frontend_library_or_framework = frontend_library_or_framework
        self.static_directory_name = "static"

        # set the shell_parameter according to the operating system of the user
        self.shell_parameter = sys.platform == constants.WINDOWS_OS_IDENTIFIER

    def validate_appname(self):
        """
        Validates the app name of the django app provided by the user via commandline
        """
        # if appname contains non alphanumeric characters then raise error and inform the user
        if not self.app_name.isalnum():
            raise CommandError(
                constants.COMMAND_ERROR_MESSAGES_DICT["INVALID_APP_NAME_ERROR_MESSAGE"]
            )

    def check_system_requirements(self):
        """
        Checks if system has node and npm installed with the help of the
        subprocess module
        """
        # command to check node in system
        command_for_node = "node -v"

        # command to check npm in system
        command_for_npm = "npm -v"

        # checks whether node is installed by getting node version
        subprocess_node_command = subprocess.run(
            shlex.split(command_for_node),
            capture_output=True,
            shell=self.shell_parameter,
        )

        # checks whether npm is installed by getting npm version
        subprocess_npm_command = subprocess.run(
            shlex.split(command_for_npm),
            capture_output=True,
            shell=self.shell_parameter,
        )

        # if either of the commands fail then raise an error and inform the user
        if (
            subprocess_node_command.returncode != 0
            or subprocess_npm_command.returncode != 0
        ):
            raise CommandError(
                constants.COMMAND_ERROR_MESSAGES_DICT["SYSTEM_ERROR_MESSAGE"]
            )

    def create_required_directories(self):
        """
        Creates templates directory and static directory inside the django
        app which will have the frontend configuration
        """

        # create the path for the templates directory
        templates_directory_path = os.path.join(
            os.getcwd(),
            self.app_name,
            constants.TEMPLATES_DIRECTORY_NAME,
            self.app_name,
        )

        # create the path for the static directory
        static_directory_path = os.path.join(
            os.getcwd(), self.app_name, self.static_directory_name, self.app_name
        )

        # create the path for the src directory
        src_directory_path = os.path.join(
            os.getcwd(), self.app_name, constants.SRC_DIRECTORY_NAME
        )

        # create the necessary directories inside the newly created django app
        try:
            os.makedirs(templates_directory_path)
            os.makedirs(static_directory_path)
            os.makedirs(src_directory_path)

        except OSError as error:
            raise CommandError(error)

    def check_if_file_is_text_document(self, filename):
        """
        Check whether file is of form textdocument or an media file with the help of file extension
        :param filename str: name of the file to check
        :return is_text_document bool: Whether the file is text document or a media file
        """

        is_text_document = True

        # get the extension of the file from the filename
        _, file_extension = os.path.splitext(filename)

        # list of non text document file extensions
        non_text_document_file_extensions = [".png"]

        # if the file extension is in the extensions list then return False
        if file_extension in non_text_document_file_extensions:
            return not is_text_document

        # return the boolean value
        return is_text_document

    def get_target_path_of_template_file(self, filename, directory_type):
        """
        Finds the path where the file after modifications
        :param filename str: name of the file to get the path
        :param directory_type str: name of the directory where file will be stored
        :return target_filepath str: final filepath of the file after it is substituted with the parameters
        """

        # initialize the target_filepath as the current working directory
        target_filepath = os.getcwd()

        # append the specific path as per the directory in which the file would be stored
        if directory_type == constants.APP_DIRECTORY_NAME:
            target_filepath = os.path.join(target_filepath, self.app_name)

        elif directory_type == constants.SRC_DIRECTORY_NAME:
            target_filepath = os.path.join(
                target_filepath, self.app_name, constants.SRC_DIRECTORY_NAME
            )

        elif directory_type == constants.TEMPLATES_DIRECTORY_NAME:
            target_filepath = os.path.join(
                target_filepath,
                self.app_name,
                constants.TEMPLATES_DIRECTORY_NAME,
                self.app_name,
            )

        # append the filename into the target_filepath
        target_filepath = os.path.join(target_filepath, filename)

        # return the target_filepath
        return target_filepath

    def download_template_files(self):
        """
        Download template files from the Git Repository and substitute the
        necessary parameters
        """

        # get all template files as per the requirement
        template_files = constants.PROD_TEMPLATE_FILES_DICT.get(
            self.frontend_library_or_framework
        )

        # replace the parameters with appropriate values
        substitute_parameters = {"app_name": self.app_name}

        # iterate all the tempate filenames
        for directory_type, filename, download_url in template_files:

            # get the path where the file will be stored
            target_filepath = self.get_target_path_of_template_file(
                filename, directory_type
            )

            # Download the files from the Git Repository
            download_file = requests.get(download_url, stream=True)

            # write the contents of the file in the appropriate location
            with open(target_filepath, "wb") as target_file:

                try:
                    target_file.write(download_file.content)

                except OSError as error:
                    raise CommandError(error)

            # if file is not a type of text document then do not perform
            # string templating for it
            if not self.check_if_file_is_text_document(filename):
                continue

            # open the file and perform templating to replace the necessary parameters
            with open(target_filepath, "r") as target_file:
                # read the file contents and substitute the parameters
                source_file = Template(target_file.read())
                modified_file_contents = source_file.substitute(substitute_parameters)

            # write the updated file in the appropriate location
            with open(target_filepath, "w") as target_file:

                try:
                    target_file.write(modified_file_contents)

                except OSError as error:
                    raise CommandError(error)

    def load_assets_from_local(self):
        """
        A development only method executed when SOFTWARE_ENVIRONMENT_MODE == development
        It loads the assets files from local system instead of downloading from the
        Git Repository
        """

        # get all template files as per the requirement
        template_files = constants.DEVELOPMENT_TEMPLATE_FILES_DICT.get(
            self.frontend_library_or_framework
        )

        # replace the parameters with appropriate values
        substitute_parameters = {"app_name": self.app_name}

        # iterate all the tempate filenames
        for directory_type, filename, local_asset_file_path in template_files:

            # get the path where the file will be stored
            target_filepath = self.get_target_path_of_template_file(
                filename, directory_type
            )

            # copy the template file from the local assets directory to the target path
            shutil.copy(local_asset_file_path, target_filepath)

            # if file is not a type of text document then do not template that file
            if not self.check_if_file_is_text_document(filename):
                continue

            # open the files and perform templating to replace the necessary parameters
            with open(target_filepath, "r") as target_file:
                source_file = Template(target_file.read())
                modified_file_contents = source_file.substitute(substitute_parameters)

            # write the updated file in the appropriate location
            with open(target_filepath, "w") as target_file:

                try:
                    target_file.write(modified_file_contents)

                except OSError as error:
                    raise CommandError(error)

    def install_dependencies(self, thread_queue):
        """
        Installs the node dependencies by executing npm install command
        :param thread_queue Queue: Queue which will pass the status of the command
        to main thread
        """
        # set the current working directory to the newly created django app
        os.chdir(self.app_name)

        # command to install node dependencies
        command_for_npm_install = "npm install"

        # execute the npm install command for installing dependencies
        command = subprocess.Popen(
            shlex.split(command_for_npm_install),
            shell=self.shell_parameter,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )

        # get any data genrated by the command and display it on stdout
        while True:

            # read output lines one line at a time
            realtime_output = command.stdout.readline()

            # if output line is empty then terminate the loop
            if realtime_output == "" or command.poll() is not None:
                break
            elif realtime_output:
                # display messages in stdout
                print(realtime_output.strip().decode())
                sys.stdout.flush()

        # set the working directory to the original value
        os.chdir("../")
        # return the status of the installation command into the main thread
        thread_queue.put(command.returncode)

    def install_dependencies_and_show_progress_bar(self):
        """
        Install the necessary node dependencies and display a progress bar
        while the installation command executes
        """
        # setting the progress bar message and type
        widgets = ["Installing Node Dependencies ", progressbar.AnimatedMarker()]

        # create an indefinite progressbar because time for installing dependencies is not equal for all users
        bar = progressbar.ProgressBar(
            widgets=widgets, max_value=progressbar.UnknownLength, redirect_stdout=True
        )
        bar.start()

        # thread_queue is used to fetch the output of the function executed in the other thread
        thread_queue = queue.Queue()
        count = 0

        # create a thread which will execute the npm install command
        thread1 = threading.Thread(
            target=self.install_dependencies, args=[thread_queue], name="thread1"
        )

        # start the installation command by starting the thread
        thread1.start()

        # display the progress until the other thread is alive
        while thread1.is_alive():
            count += 1
            bar.update(count)

        # fetching status code of the command executed in other thread
        status = thread_queue.get()

        # if command failed then inform the user
        if status != 0:
            raise CommandError(
                constants.COMMAND_ERROR_MESSAGES_DICT["NPM_INSTALLATION_ERROR_MESSAGE"]
            )

    def generate(self):
        """
        Generates a django app with the selected frontend library or framework
        Driver method which will execute the other methods
        """
        self.validate_appname()
        self.check_system_requirements()
        management.call_command("startapp", self.app_name)
        self.create_required_directories()

        # In development mode the assets will be loaded from the local system
        if os.environ.get("SOFTWARE_ENVIRONMENT_MODE") == "development":
            print("Loading the Assets Locally")
            self.load_assets_from_local()

        # In production mode the assets will be downloaded from the Git Repository
        else:
            self.download_template_files()

        self.install_dependencies_and_show_progress_bar()



APP_DIRECTORY_NAME = "APP"
SRC_DIRECTORY_NAME = "src"
TEMPLATES_DIRECTORY_NAME = "templates"

# Production React Ts Template files having Github Repository Links
REACTTS_TEMPLATES_URLS_DICT = {
    "package.json-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/package.json-tpl",
    "webpack.config.js-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/webpack.config.js-tpl",
    "babel.config.json-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/babel.config.json-tpl",
    "tsconfig.json-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/tsconfig.json-tpl",
    "module.d.ts-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/module.d.ts-tpl",
    "App.tsx-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/App.tsx-tpl",
    "index.tsx-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/index.tsx-tpl",
    "App.css-tpl": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/App.css-tpl",
    "reactlogo.png": "https://raw.githubusercontent.com/Jitensid/django-webpack-dev-server/main/assets/reactts/reactlogo.png",
}

# Template files specific to react ts configuration
PROD_REACTTS_TEMPLATE_FILES = [
    (
        APP_DIRECTORY_NAME,
        "package.json",
        REACTTS_TEMPLATES_URLS_DICT["package.json-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "webpack.config.js",
        REACTTS_TEMPLATES_URLS_DICT["webpack.config.js-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "babel.config.json",
        REACTTS_TEMPLATES_URLS_DICT["babel.config.json-tpl"],
    ),
    (
        APP_DIRECTORY_NAME,
        "tsconfig.json",
        REACTTS_TEMPLATES_URLS_DICT["tsconfig.json-tpl"],
    ),
    (SRC_DIRECTORY_NAME, "module.d.ts", REACTTS_TEMPLATES_URLS_DICT["module.d.ts-tpl"]),
    (SRC_DIRECTORY_NAME, "App.tsx", REACTTS_TEMPLATES_URLS_DICT["App.tsx-tpl"]),
    (SRC_DIRECTORY_NAME, "index.tsx", REACTTS_TEMPLATES_URLS_DICT["index.tsx-tpl"]),
    (SRC_DIRECTORY_NAME, "App.css", REACTTS_TEMPLATES_URLS_DICT["App.css-tpl"]),
    (SRC_DIRECTORY_NAME, "reactlogo.png", REACTTS_TEMPLATES_URLS_DICT["reactlogo.png"]),
]
